#rm(list=ls())
library(qtl)
library(ASMap)
library(zoo)
library(stringr)
library(roll)
source("00_source_functions.R")

# --- Fal specific parameters --- # 
#map <- read.cross(format="csv", file="FalF1map_bwa_GATK_full_table.txt", na.strings="NA", genotypes=c("A","H"), estimate.map=F)
#switchlist1 <- c("L.1", "L.10", "L.1021", "L.1087", "L.1109", "L.1112", "L.125", "L.1265", "L.1284", "L.1304", "L.1306", "L.1431", "L.1467", "L.15", "L.1476", "L.369", "L.393", "L.41", "L.420", "L.487", "L.755", "L.860")
#switchlist2 <- c("L.128", "L.1412", "L.244", "L.424", "L.528", "L.995")
#LGs_todrop <- c("L.259.2.3", "L.272.2", "UL1.3", "UL2.3", "UL30.2")
#R_out <- "02_data/Fal_full.RData"
#markers_out <- "02_data/Fal_full.txt"

# --- Pet specific parameters --- # 
#map <- read.cross(format="csv", file="-PetF1map_bwa_GATK_full_table.txt", na.strings="NA", genotypes=c("A","H"), estimate.map=F)
#switchlist1 <- c("L.1", "L.101", "L.1042", "L.1048", "L.124", "L.177", "L.213", "L.223", "L.253", "L.327", "L.36", "L.431", "L.447", "L.667", "L.77", "L.897")
#switchlist2 <- c("L.2", "L.28", "L.451", "L.53", "L.56", "L.575", "L.422", "L.448", "L.763", "L.816", "L.832", "L.838", "L.898", "L.994", "L.977")
#LGs_todrop <- c("L.282.4.7", "L.283.2.13.2", "UL1.5", "UL2.2", "UL102")
#R_out <- "02_data/Pet_full.RData"
#markers_out <- "02_data/Pet_full.txt"

# Remove individuals with fewer than 50% markers genotyped & markers with fewer than 50% individuals genotyped & extreme segregation patterns
#plot(ntyped(map), ylab="No. typed markers", main="No. genotypes by individual")
map <- subset(map, ind=(ntyped(map)>max(ntyped(map))*0.5)) 
#plot(ntyped(map, "mar"), ylab="No. typed individuals", main="No.genotypes by marker")
map <- drop.markers(map, names(ntyped(map, "mar")[ntyped(map, "mar") < nrow(map[[2]])*0.5]))
#profileMark(map, stat.type="prop")
gt <- geno.table(map)
gt$prop <- gt$AA/(gt$AB+gt$AA)
map <- drop.markers(map, row.names(gt[(gt$prop<0.2 | gt$prop>0.8),]))

# Put duplicate markers aside
#dup1 <- findDupMarkers(map, exact.only=F)
#save(dup1, file="02_data/Fdup1.RData")
#save(dup1, file="02_data/Pdup1.RData")
map <- drop.markers(map, unlist(dup1)) 

# Make map with a p-value for which none of the LGs look like they are doubled up 
map <- mstmap.cross(map, id="NA.", bychr=FALSE, dist.fun="kosambi", trace=TRUE, p.value=1e-18)
plotMap(map, show.marker.names=F)

# Flip alleles in linkage groups with decreasing size
for (i in seq(20, 2, by = -1)) {
  print(i)
  map <- switchAlleles(map, markernames(map, chr=row.names(summaryMap(map)[summaryMap(map)$n.mar<i,])))
  map <- mstmap.cross(map, id="NA.", bychr=FALSE, dist.fun="kosambi", trace=TRUE, p.value=1e-18)
}

# Switch LGs with high recombination fractions and high lod scores
#plotRF(map, alternate.chrid=TRUE, chr=row.names(summaryMap(map)[(summaryMap(map)$n.mar>3 & summaryMap(map)$n.mar<75),]))
#plotRF(map, alternate.chrid=TRUE, chr=row.names(summaryMap(map)[summaryMap(map)$n.mar>50,]))
map <- switchAlleles(map, markernames(map, chr=switchlist1))
map <- mstmap.cross(map, id="NA.", bychr=FALSE, dist.fun="kosambi", trace=TRUE, p.value=1e-18)
#save(map, file="02_data/Ftemp1.RData")
#save(map, file="02_data/Ptemp1.RData")
#plotRF(map, alternate.chrid=TRUE, chr=row.names(summaryMap(map)[summaryMap(map)$n.mar>3,]))
map <- switchAlleles(map, markernames(map, chr=switchlist2))

# Put duplicate markers aside
dup2 <- findDupMarkers(map, exact.only=F)
#save(dup2, file="02_data/Fdup2.RData")
#save(dup2, file="02_data/Pdup2.RData")
map <- drop.markers(map, unlist(dup2))

# Move more questionable markers to other dataset 
map <- pullCross(map, type = "seg.distortion", pars = list(seg.thresh = 0.05))
map <- pullCross(map, type = "missing", pars = list(miss.thresh = 0.1))

# Make map & drop linkage groups with only 2 markers
map <- mstmap(map, id="NA.", bychr=FALSE, dist.fun="kosambi", trace=TRUE, p.value=1e-6, noMap.size = 2, noMap.dist = 5)
map <- drop.markers(map, markernames(map, chr=c(row.names(summaryMap(map)[summaryMap(map)$n.mar<4,]))))
plotMap(map, show.marker.names=F)
#plotRF(map, alternate.chrid=TRUE)

# Iteratively remove questionable markers and tiny LGs
#profileMark(map, stat.type=c("seg.dist", "prop", "dxo", "recomb", "miss"), layout=c(1, 6), type="l")
map <- dxo_remover(map, 3, TRUE, 1e-6)
map <- sd_outlier_remover(map, 21, 2, TRUE, 1e-6)
map <- drop.markers(map, markernames(map, chr=c(row.names(summaryMap(map)[summaryMap(map)$n.mar<4,]))))

# Push some markers back into map
map <- pushCross(map, type = "missing", pars = list(miss.thresh = 0.3))
map <- pushCross(map, type = "seg.distortion", pars = list(seg.thresh = 0.01))
map <- mstmap(map, id="NA.", bychr=TRUE, dist.fun="kosambi", trace=TRUE, p.value=1e-6, noMap.size = 2, noMap.dist = 5)

# Iteratively remove questionable markers and tiny LGs
map <- dxo_remover(map, 3, TRUE, 1e-6)
map <- sd_outlier_remover(map, 21, 2, TRUE, 1e-6)
map <- drop.markers(map, markernames(map, chr=c(row.names(summaryMap(map)[summaryMap(map)$n.mar<4,]))))

# Push more markers back into map
map <- pushCross(map, type = "missing", pars = list(miss.thresh = 0.5))
map <- pushCross(map, type = "seg.distortion", pars = list(seg.thresh = 0.001))
map <- mstmap(map, id="NA.", bychr=TRUE, dist.fun="kosambi", trace=TRUE, p.value=1e-6, noMap.size = 2, noMap.dist = 5)

# Iteratively remove questionable markers and tiny LGs
map <- dxo_remover(map, 3, TRUE, 1e-6)
map <- sd_outlier_remover(map, 21, 2, TRUE, 1e-6)
map <- drop.markers(map, markernames(map, chr=c(row.names(summaryMap(map)[summaryMap(map)$n.mar<4,]))))

# Look for problems with map
plotMap(map, show.marker.names=F)
profileMark(map, stat.type=c("seg.dist", "prop", "dxo", "recomb", "miss"), layout=c(1, 6), type="l")
plotRF(map, alternate.chrid=TRUE)

#save(map, file="02_data/Ftemp2.RData")
#save(map, file="02_data/Ptemp2.RData")

# Remove genotypes that are likely to be errors questionable markers and tiny LGs
# Repeat lines until there are no more errors to fix (3+ times)
map <- calc.errorlod(map)
toperr <- top.errorlod(map, cutoff=2)
for(i in 1:nrow(toperr)) { 
  chr <- toperr$chr[i]
  id <- toperr$id[i]
  mar <- toperr$marker[i]
  map$geno[[chr]]$data[id, mar] <- NA 
}
map <- dxo_remover(map, 3, TRUE, 1e-6)
map <- sd_outlier_remover(map, 21, 2, TRUE, 1e-6)
map <- drop.markers(map, markernames(map, chr=c(row.names(summaryMap(map)[summaryMap(map)$n.mar<4,]))))

#save(map, file="02_data/Ftemp3.RData")
#save(map, file="02_data/Ptemp3.RData")

# Drop extra LGs and look at and save map data
map <- drop.markers(map, markernames(map, chr=LGs_todrop))
map <- mstmap(map, id="NA.", bychr=TRUE, dist.fun="kosambi", trace=TRUE, p.value=1e-6, noMap.size = 2, noMap.dist = 5, detectBadData = TRUE)
plotMap(map, show.marker.names=F, las=2)
#plotRF(map, alternate.chrid=TRUE)
summaryMap(map)
save(map, file = R_out)

# Pull out markers
markers <- c()
pos <- c()
chrom <- c()
for(i in chrnames(map)) {
  data <- pull.map(map, i)
  markers <- c(markers, names(data[[1]]))
  pos <- c(pos, as.numeric(data[[1]]))
  chrom <- c(chrom, rep(i, length(data[[1]])))
}
map.data <- data.frame(markers, chrom, pos)

#load("02_data/Fdup1.RData")
#load("02_data/Fdup2.RData")
#load("02_data/Pdup1.RData")
#load("02_data/Pdup2.RData")

## Add duplicates into the map
dup.list <- list(dup2, dup1)
for(d in dup.list){
  duplicates <- c()
  for(j in 1:length(d)){
    for(i in (1:length(d[[j]]))){
      if (sum(map.data$markers==names(d[j]))==1){
        duplicates <- rbind(duplicates,(cbind(d[[j]][i],map.data[map.data$markers==names(d[j]),2:3])))
      }
    }
  }
  names(duplicates) <- c("markers", "chrom", "pos")
  map.data <- rbind(map.data, duplicates)
}

write.table(map.data, file = markers_out, sep="\t", row.names=F, quote=F)

##########################################################
### --- Explore why Fal (Neg) chr2 and chr4 are so short --- ###
##########################################################

# Note: at the very begining there are markers as many chr2 and chr4 markers
# head -1 NegF1map.bwa.GATK.full.table.txt | tr , '\n' | grep Chr01 | wc
# after building the maps there are 156 Fal chr02 and 166 chr04 while the other chromosomes range from 326-913
# grep Chr01 Fal_full.txt | wc

library(dplyr)
map <- read.cross(format="csv", file="01_data/FalF1map.bwa.GATK.full.table.txt", na.strings="NA", genotypes=c("A","H"), estimate.map=F)
ntyped(map, "mar")
gt <- geno.table(map)
gt$prop <- gt$AB/(gt$AB+gt$AA)
gt$markers <- rownames(gt)
gt$ann.chr <- gt$markers %>% gsub("HanXRQChr", "", .) %>% gsub("_.*", "", .)
gt <- filter(gt, ann.chr != "00c0041")
boxplot(gt$prop ~ gt$ann.chr, las = 2, xlab = "H. annuus chromosomes", ylab = "Marker heterozygosity", main = "H. annuus x H. petiolaris ssp. fallax cross")
# chrs2 and chr4 have extreme segregation distortion (almost fixed AB)

library(ggplot2)
ggplot(gt, aes(x=ann.chr, y=prop)) + geom_violin()
gt$ann.pos <- gt$markers %>% gsub("HanXRQChr.._", "", .) %>% as.numeric()

for (i in names(table(gt$ann.chr))) {
  temp <- gt %>% filter(ann.chr==i)
  plot <- qplot(temp$ann.pos, temp$prop, geom = c('point', 'smooth'), ylim = c(0,1)) + theme_classic()
  print(plot)
}

# There are fewer informative markers on one end of each of chromosomes 2 and 4 - this could be because there is less heterogygostiy in those regions. 
# Perhaps because of a recent selective sweep or lots of background selection
# A higher proportion of markers that appear to have fixed differences between Ann and Fal
