rm(list=ls())
library(qtl)
library(ASMap)
library(zoo)
library(stringr)
library(roll)
source("00_source_functions.R")

# --- Fal specific parameters --- # 
#map <- read.cross(format="csv", file="FalF1map_bwa_GATK_thin_table.txt", na.strings="NA", genotypes=c("A","H"), estimate.map=F)
#switchlist1 <- c("L.1", "L.101", "L.120", "L.127", "L.131", "L.133", "L.155", "L.167", "L.195", "L.216", "L.218", "L.272", "L.290", "L.295", "L.305", "L.373",  "L.376", "L.456", "L.473", "L.504")
#switchlist2 <- c("L.1", "L.124", "L.142", "L.217")
#LGs_todrop <- c("UL1.2")
#R_out <- "02_data/Fal_thin.RData"
#markers_out <- "02_data/Fal_thin.txt"

# --- Pet specific parameters --- # 
#map <- read.cross(format="csv", file="-PetF1map_bwa_GATK_thin_table.txt", na.strings="NA", genotypes=c("A","H"), estimate.map=F)
#switchlist1 <- c("L.1", "L.127", "L.145", "L.146", "L.172", "L.176", "L.181", "L.200", "L.201", "L.218", "L.25", "L.250", "L.253", "L.254", "L.34", "L.88")
#switchlist2 <- c("L.32")
#LGs_todrop <- c("L.154.3")
#R_out <- "02_data/Pet_thin.RData"
#markers_out <- "02_data/Pet_thin.txt"

# Remove individuals with fewer than 50% markers genotyped & markers with fewer than 50% individuals genotyped & extreme segregation patterns
#plot(ntyped(map), ylab="No. typed markers", main="No. genotypes by individual")
map <- subset(map, ind=(ntyped(map)>max(ntyped(map))*0.5)) 
#plot(ntyped(map, "mar"), ylab="No. typed individuals", main="No.genotypes by marker")
map <- drop.markers(map, names(ntyped(map, "mar")[ntyped(map, "mar") < nrow(map[[2]])*0.5]))
#profileMark(map, stat.type="prop")
gt <- geno.table(map)
gt$prop <- gt$AA/(gt$AB+gt$AA)
map <- drop.markers(map, row.names(gt[(gt$prop<0.2 | gt$prop>0.8),]))

# Make map with a p-value for which none of the LGs look like they are doubled up 
map <- mstmap.cross(map, id="NA.", bychr=FALSE, dist.fun="kosambi", trace=TRUE, p.value=1e-16)
plotMap(map, show.marker.names=F)

# Flip alleles in linkage groups with decreasing size
for (i in seq(20, 2, by = -1)) {
  map <- switchAlleles(map, markernames(map, chr=row.names(summaryMap(map)[summaryMap(map)$n.mar<i,])))
  map <- mstmap.cross(map, id="NA.", bychr=FALSE, dist.fun="kosambi", trace=TRUE, p.value=1e-16)
}

# Switch LGs with high recombination fractions and high lod scores
#plotRF(map, alternate.chrid=TRUE, chr=row.names(summaryMap(map)[(summaryMap(map)$n.mar>2 & summaryMap(map)$n.mar<75),]))
#row.names(summaryMap(map)[(summaryMap(map)$n.mar>2 & summaryMap(map)$n.mar<75),])
#plotRF(map, alternate.chrid=TRUE, chr=row.names(summaryMap(map)[summaryMap(map)$n.mar>50,]))
#row.names(summaryMap(map)[summaryMap(map)$n.mar>50,])
map <- switchAlleles(map, markernames(map, chr=switchlist1))
map <- mstmap.cross(map, id="NA.", bychr=FALSE, dist.fun="kosambi", trace=TRUE, p.value=1e-16)
#plotRF(map, alternate.chrid=TRUE, chr=row.names(summaryMap(map)[summaryMap(map)$n.mar>2,]))
map <- switchAlleles(map, markernames(map, chr=switchlist2))

# Move more questionable markers to other dataset 
map <- pullCross(map, type = "co.located")
map <- pullCross(map, type = "seg.distortion", pars = list(seg.thresh = 0.05))
map <- pullCross(map, type = "missing", pars = list(miss.thresh = 0.1))

# Make map & drop linkage groups with only 2 markers
map <- mstmap(map, id="NA.", bychr=FALSE, dist.fun="kosambi", trace=TRUE, p.value=1e-6, noMap.size = 2, noMap.dist = 5)
map <- drop.markers(map, markernames(map, chr=c(row.names(summaryMap(map)[summaryMap(map)$n.mar<4,]))))
plotMap(map, show.marker.names=F)
#plotRF(map, alternate.chrid=TRUE)

# Iteratively remove questionable markers and tiny LGs
#profileMark(map, stat.type=c("seg.dist", "prop", "dxo", "recomb", "miss"), layout=c(1, 6), type="l")
map <- dxo_remover(map, 3, TRUE, 1e-6)
map <- sd_outlier_remover(map, 21, 2, TRUE, 1e-6)
map <- drop.markers(map, markernames(map, chr=c(row.names(summaryMap(map)[summaryMap(map)$n.mar<4,]))))

# Push some markers back into map
map <- pushCross(map, type = "missing", pars = list(miss.thresh = 0.3))
map <- pushCross(map, type = "seg.distortion", pars = list(seg.thresh = 0.01))
map <- mstmap(map, id="NA.", bychr=TRUE, dist.fun="kosambi", trace=TRUE, p.value=1e-6, noMap.size = 2, noMap.dist = 5)

# Iteratively remove questionable markers and tiny LGs
map <- dxo_remover(map, 3, TRUE, 1e-6)
map <- sd_outlier_remover(map, 21, 2, TRUE, 1e-6)
map <- drop.markers(map, markernames(map, chr=c(row.names(summaryMap(map)[summaryMap(map)$n.mar<4,]))))

# Push more markers back into map
map <- pushCross(map, type = "missing", pars = list(miss.thresh = 0.5))
map <- pushCross(map, type = "seg.distortion", pars = list(seg.thresh = 0.001))
map <- mstmap(map, id="NA.", bychr=TRUE, dist.fun="kosambi", trace=TRUE, p.value=1e-6, noMap.size = 2, noMap.dist = 5)

# Iteratively remove questionable markers and tiny LGs
map <- dxo_remover(map, 3, TRUE, 1e-6)
map <- sd_outlier_remover(map, 21, 2, TRUE, 1e-6)
map <- drop.markers(map, markernames(map, chr=c(row.names(summaryMap(map)[summaryMap(map)$n.mar<4,]))))

# Look for problems with map
plotMap(map, show.marker.names=F)
profileMark(map, stat.type=c("seg.dist", "prop", "dxo", "recomb", "miss"), layout=c(1, 6), type="l")
plotRF(map, alternate.chrid=TRUE)

# Push final markers back into map
map <- pushCross(map, type = "co.located")

# Remove genotypes that are likely to be errors questionable markers and tiny LGs
# Repeat lines until there are no more errors to fix (3 or 4 times)
map <- calc.errorlod(map)
toperr <- top.errorlod(map, cutoff=2)
for(i in 1:nrow(toperr)) { 
  chr <- toperr$chr[i]
  id <- toperr$id[i]
  mar <- toperr$marker[i]
  map$geno[[chr]]$data[id, mar] <- NA 
}
map <- dxo_remover(map, 3, TRUE, 1e-6)
map <- sd_outlier_remover(map, 21, 2, TRUE, 1e-6)
map <- drop.markers(map, markernames(map, chr=c(row.names(summaryMap(map)[summaryMap(map)$n.mar<4,]))))

# Drop extra LGs and look at and save map data
map <- drop.markers(map, markernames(map, chr=LGs_todrop))
plotMap(map, show.marker.names=F)
#plotRF(map, alternate.chrid=TRUE)
summaryMap(map)
save(map, file = R_out)

# Pull out markers and save
markers <- c()
pos <- c()
chrom <- c()
for(i in chrnames(map)) {
  data <- pull.map(map, i)
  markers <- c(markers, names(data[[1]]))
  pos <- c(pos, as.numeric(data[[1]]))
  chrom <- c(chrom, rep(i, length(data[[1]])))
}
map.data <- data.frame(markers, chrom, pos)
write.table(map.data, file = markers_out, sep="\t", row.names=F, quote=F)
